const { EnvironmentPlugin } = require('webpack');
const mix = require('laravel-mix');
const path = require('path');
const glob = require('glob');

const vuexy = {
    mixOptions: {
        processCssUrls: false,
        postCss: [require('autoprefixer')],
        resourceRoot: process.env.ASSET_URL || undefined
    },
    pluginOptions: {
        sassOptions: {
            precision: 5
        }
    },
    webpackOptions: {
        output: {
            publicPath: process.env.ASSET_URL || undefined,
            libraryTarget: 'umd'
        },
        plugins: [
            new EnvironmentPlugin({
                BASE_URL: process.env.ASSET_URL ? `${process.env.ASSET_URL}/` : '/'
            })
        ],
        module: {
            rules: [
                {
                    test: /\.es6$|\.js$/,
                    include: [
                        path.join(__dirname, 'node_modules/bootstrap/'),
                        path.join(__dirname, 'node_modules/popper.js/'),
                        path.join(__dirname, 'node_modules/shepherd.js/')
                    ],
                    loader: 'babel-loader',
                    options: {
                        presets: [['@babel/preset-env', { targets: 'last 2 versions, ie >= 10' }]],
                        plugins: [
                            '@babel/plugin-transform-destructuring',
                            '@babel/plugin-proposal-object-rest-spread',
                            '@babel/plugin-transform-template-literals'
                        ],
                        babelrc: false
                    }
                }
            ]
        },
        externals: {
            jquery: 'jQuery',
            moment: 'moment',
            jsdom: 'jsdom',
            velocity: 'Velocity',
            hammer: 'Hammer',
            pace: '"pace-progress"',
            chartist: 'Chartist',
            'popper.js': 'Popper',
            // blueimp-gallery plugin
            './blueimp-helper': 'jQuery',
            './blueimp-gallery': 'blueimpGallery',
            './blueimp-gallery-video': 'blueimpGallery'
        }
    },
    assets: (query, callback, ignore = [], source = 'resources/assets', output = 'public/assets') => {
        let options = {
            ignore: []
        };

        if (ignore.length > 0) {
            ignore.forEach(path => {
                options.ignore.push(`${source}/${path}`);
            });
        }

        (glob.sync(`${source}/${query}`, options) || []).forEach(file => {
            file = file.replace(/[\\\/]+/g, '/');
            callback(file, file.replace(source, output));
        });
    },
    output: (input) => input.replace(/([\\/])scss([\\/])/, '$1css$2').replace(/\.scss$/, '.css'),
    copy: (from, to) => mix.copy(from, to),
    sass: (src, output) => mix.sass(src, vuexy.output(output), vuexy.pluginOptions),
    version: (files = []) => mix.version(files),
    disableNotifications: () => mix.disableNotifications(),
    options: () => mix.options(vuexy.mixOptions),
    webpackConfig: () => mix.webpackConfig(vuexy.webpackOptions),
    js: (src, output) => mix.js(src, output),
    scripts: (src, output) => mix.scripts(src, output),
    copyDirectory: (src, output) => mix.copyDirectory(src, output),
    browserSync: (proxy) => mix.browserSync(proxy),
    init: () => {
        vuexy.version();
        vuexy.disableNotifications();
        vuexy.options();
        vuexy.webpackConfig();
    }
};

module.exports = vuexy;
